function [r,gofr,q,sofq] = fft_1d_real
%   This code creates a one dimensional vector of data points 
%   that is input for a Fast Fourier Transform calculation
%
%   Input function in this file.
% 
%   Requires realft.m (from Numerical Recipes)
%
%   sample usage:  [r,gofr,q,sofq] = fft_1d_real
%
%   code author:  David J. Keffer
%   University of Tennessee
%   dkeffer@utk.edu
%   code created:  August, 2006
%   code updated:  March, 2014
%    
clear all;
close all;
%
    m = 12;
%   number of points 
    n = 2^m;
%
%   dimension and initialize variables
%
%   r:  position vector in real space
r = zeros(1,n);
%   gofr:  pair correlation function
gofr = zeros(1,n);
%   q:  position vector in reciprocal space
q = zeros(1,n);
%   sofq:  structure factor
sofq = zeros(1,n);
%
%  isign = -1 for transformation from r to q
%  isign =  1 for transformation from q to r
isign = -1;
%     define some constants
pi = 2.00*asin(1.00);
twopi = 2.00*pi;
%
% define real space vector
%
%     first point
r(1) = 0.0;
%     delta r
dr = 0.01;
%     all other points
for i = 2:1:n
   r(i) = r(i-1) + dr;
end
%
%     define sample g(r)
%     this is a made up example of a sum of cosines
%
ncos = 5;
c = zeros(1,ncos);
t = zeros(1,ncos);
c(1) = 1.0;
c(2) = 2.0;
c(3) = 1.0;
c(4) = 2.0;
c(5) = 1.0;
t(1) = 1.00;
t(2) = 0.50;
t(3) = 2.00;
t(4) = 4.00;
t(5) = 10.00;
%
for i = 1:1:n
   for j = 1:1:ncos
      gofr(i) = gofr(i) + c(j)*cos(r(i)*2.0*pi/t(j));
   end
end
%
%     get FFT
%
sofq = realft(gofr,n,isign);
%
%     determine k-space coordinates
%
deni = 1.00/(2.0*n*dr);
for i = 1:1:n
   ii = i - 1;
   q(i) = ii*deni;
end
%
%     save results to file
%
fid = fopen('fft_1d_real_out.txt','w');
fprintf(fid,'r                gofr                   q                       sofq \n');
for i = 1:1:n
fprintf(fid,'%23.15e %23.15e %23.15e %23.15e   \n', r(i),gofr(i),q(i),sofq(i));
end
fclose(fid);
%
%  plot original signal
%
figure (1)
plot (r,gofr,'k-');
xlabel( 'r' );
ylabel ( 'g(r)' );

%
%  plot fft
%
figure (2)
plot (q,sofq,'k-');
xlabel( 'q' );
ylabel ( 's(q)' );


%
%  plot fft close-up
%
figure (3)
plot (q,sofq,'k-');
xlabel( 'q' );
ylabel ( 's(q)' );
ymin = min(sofq);
ymax = max(sofq);
axis([0 5 ymin ymax]);


%
%  perform inverse fft to reclaim original signal
%
figure(4)
gofr2= realft(sofq,n,-isign);
plot (r,real(gofr2),'k-');
hold on;
plot (r,imag(gofr2),'r-');
hold off;
xlabel( 'r' );
ylabel ( 'g(r)' );
legend('real','imaginary')

%
%  perform  fft using Matlab intrinsic function
%

%
%     determine k-space coordinates
%
deni = 1.00/(n*dr);
for i = 1:1:n
   ii = i - 1;
   q2(i) = ii*deni;
end
sofq2 = fft(gofr,n);

figure(5)
plot (q2,real(sofq2),'k-');
hold on;
plot (q2,imag(sofq2),'r-');
hold off;
xlabel( 'q' );
ylabel ( 's(q)' );
legend('real','imaginary')

%
%  plot fft close-up
%
figure (6)
plot (q2,sofq2,'k-');
xlabel( 'q' );
ylabel ( 's(q)' );
ymin = min(real(sofq2));
ymax = max(real(sofq2));
axis([0 5 ymin ymax]);

%
%  perform inverse fft to reclaim original signal
%
figure(7)
gofr3 = ifft(sofq2,n);
plot (r,real(gofr3),'k-');
hold on;
plot (r,imag(gofr3),'r-');
hold off;
xlabel( 'r' );
ylabel ( 'g(r)' );
legend('real','imaginary')